@@ -12,7 +12,7 @@
 *  All changes concern only the program interface, so this modified
 *  version produces exactly the same instances as the original version.
 *
-*  Changes were made by Andrew Makhorin <mao@mai2.rcnet.ru>.
+*  Changes were made by Andrew Makhorin <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 ***********************************************************************/
 
 #include "glpapi.h"
-#include "glprng.h"
+#include "rng.h"
 
 /***********************************************************************
 *  NAME
